@@ -13,19 +13,8 @@
 #
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-import os
 
-if os.path.exists("/tmp/openvpn-applet.debug"):
-    enable_logging = True
-else:
-    enable_logging = False
-
-if enable_logging:
-    fd = open('/tmp/openvpn-applet.log','w+')
-else:
-    fd = None
-
-app_version = '0.6.0'
+app_version = '0.6.1'
 about_name = 'OpenVPN Applet'
 about_text = 'Applet for controlling OpenVPN connections'
 about_authors = 'Mikko Vartiainen'
@@ -39,20 +28,20 @@ donate_device_url = ''
 import sys
 sys.path.insert(0, '/usr/lib/openvpn-applet')
 
-import glob
-import string
+import debug
+log = debug.log
+
 from subprocess import Popen, PIPE, call
 import gtk
-import hildondesktop
+from hildondesktop import StatusMenuItem
 import hildon
-import gobject
-from sets import Set
-import fileinput
+from gobject import timeout_add
 import re
 from UserDict import UserDict
-import socket
-import time
 import conic
+import os
+
+import management
 
 def copy_file(file):
     log("Copy: ", file)
@@ -90,102 +79,39 @@ def openvpn_running(pid):
 class vpn():
     def __init__(self, conf_file, conf_dir='/etc/openvpn', auth_callback_fb=None):
         self.conf_file = conf_file
-        self.name = string.split(os.path.basename(self.conf_file),'.')[0]
+        self.name = os.path.basename(self.conf_file).split('.')[0]
         self.pid_file = '/var/run/openvpn.'+self.name+'.pid'
+        self.pid = None
         self.conf_dir = conf_dir
         self.stopcmd = '/etc/init.d/openvpn'
         self.startcmd = '/usr/sbin/openvpn'
         self.socket_name = '/tmp/'+self.name+'.socket'
-        self.socket = None
+        self.management = management.Connection(self.socket_name)
         self.auth_callback_fb = auth_callback_fb
 
-    def __socket_connect(self):
-        if self.socket == None:
-            self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
-            try:
-                self.socket.connect(self.socket_name)
-            except socket.error, (value,message):
-                log("socket.connect() failed:",value, message)
-                self.socket = None
-                raise socket.error(value,message);
-                return False
-            return True
-        return False
-
-    def __socket_disconnect(self):
-        if self.socket != None:
-            self.socket.close()
-            self.socket = None
-
-    def __admin_command(self, command, get_response = True, disconnect = True, wait_for_ack = True):
-        data = ''
-        if self.socket == None: 
-            if self.__socket_connect() == False:
-                return data
-        bytes_sent = self.socket.send(command)
-        log("Command: "+command,bytes_sent)
-        if (get_response):
-            data = self.__admin_recv(wait_for_ack)
-            log("Response:", data[-3:])
-
-        if disconnect:
-            self.__socket_disconnect()
-        return data
-
-    def __admin_recv(self, wait_for_ack=False):
-        if self.socket == None: return
-
-        data = ''
-        while True:
-            chunk = self.socket.recv(4096)
-            #log(chunk)
-            data = data+chunk
-            lines = data.split("\r\n")
-            if wait_for_ack:
-                if data.find("SUCCESS") != -1: break
-                if data.find("ERROR") != -1: break
-                if data.find("END") != -1: break
-                if data.find(">FATAL") != -1: break
-            else:
-                if lines[-1] == '': break
-        return lines
-
     def get_log(self):
-        data = "\n".join(self.__admin_command("log all\n")[1:])
-        while True:
-            mindex = data.find("MANAGEMENT")
-            if mindex == -1: break
-            end = data.find("\n",mindex)
-            if end == -1: end = len(data)
-            start = data.rfind("\n",0,mindex)
-            if start >= mindex or start == -1: start = 0
-            data = data[:start+1]+data[end+1:]
-    
-        return data[:-4]
+        return self.management.get_log()
 
     def get_statistics(self):
-        data = self.__admin_command("status\n")
-        return "\n".join(data[1:-2])
+        return self.management.get_statistics()
 
     def get_state(self):
-        data = self.__admin_command("state all\n")
-        return "\n".join(data[1:-2])
-        
-    def get_status(self):
+        return self.management.get_state()
+
+    def is_running(self):
         pid = self.get_pid()
                 
         if pid != None and pid_exists(pid) and openvpn_running(pid):
-            #self.pid = pid
             return True
         else:
             return False
         
     def get_pid(self):
+        if self.pid != None: return self.pid
         try:
             f = open(self.pid_file)
         except IOError:
             return None
-        
         pid = int(f.readline())    
         return int(pid)
         
@@ -199,64 +125,92 @@ class vpn():
         input = process.stdin
         input.close()
 
-        for i in range(1,10):
-            try:
-                self.__admin_command("hold off\n")
-            except socket.error:
-                time.sleep(0.2)
-            else:
-                break
-        response = self.__admin_command("hold release\n", disconnect=False)
+        status = "SUCCESS"
+        message = ""
+        (status,message) = self.management.connect()
         auth_entered = []
-        for i in range(2):
-            response.extend(self.__admin_command("test 1\n", disconnect=False, wait_for_ack=False))
-            response.extend(self.__admin_command("test 1\n", disconnect=True, wait_for_ack=False))
-            for line in response:
-                if line[:9] == ">PASSWORD":
-                    if len(line.split()) > 3:
-                        (a, auth_name,auth_name2, auth_type) = line.split()
-                        auth_name = auth_name+" "+auth_name2
-                    else:
-                        (a, auth_name, auth_type) = line.split()
-                    auth_name = auth_name[1:-1]
+        for i in range(5):
+            (auth_name, auth_type) = self.management.poll_password_request()
+            if auth_name != "ERROR":
+                ask_username = False
+
+                if auth_type == 'username/password':
+                    ask_username = True
+                    log("auth-user-pass", auth_name)
+                if auth_type == 'password':
                     ask_username = False
+                    log("askpass", auth_name)
 
-                    if auth_type == 'username/password':
-                        ask_username = True
-                        log("auth-user-pass", auth_name)
-                    if auth_type == 'password':
-                        ask_username = False
-                        log("askpass", auth_name)
+                try:
+                    auth_entered.index(auth_name)
+                except ValueError:
+                    (username,password,response_ok) = self.auth_callback_fb(ask_username, "OpenVPN Authentication: "+self.name+" "+auth_name)
+                    auth_entered.append(auth_name)
+                    if response_ok:
+                        if ask_username: self.management.username(auth_name,username)
+                        (status, message) = self.management.password(auth_name,password)
+                    else:
+                        self.stop()
+            else:
+                if len(auth_entered) > 0: break
 
-                    try:
-                        auth_entered.index(auth_name)
-                    except ValueError:
-                        (username,password,response_ok) = self.auth_callback_fb(ask_username, "OpenVPN Authentication: "+self.name+" "+auth_name)
-                        auth_entered.append(auth_name)
-                        if response_ok:
-                            if ask_username: self.__admin_command("username \""+auth_name+"\" \""+username+"\"\n")
-                            self.__admin_command("password \""+auth_name+"\" \""+password+"\"\n")
-                        else:
-                            self.stop()
+        (code, pid) = self.management.get_pid()
+        if code == "SUCCESS": self.pid = pid
+        return (status,message)
 
-        return process.returncode
-        
     def stop(self):
         log("stop")
-        self.__admin_command("signal SIGTERM\n")
+        (status, message) = self.management.disconnect()
+        if status != "SUCCESS":
+            log("using backup stop command")
+            call(['sudo', self.stopcmd, 'stop', self.name])
+        self.pid = None
 
     def restart(self):
         log("restart")
-        self.__admin_command("signal SIGUSR1\n")
+        self.management.restart()
 
     def get_conf_file(self):
-        return self.conf_file        
+        return self.conf_file
 
     def get_name(self):
         return self.name
 
+    def get_current_status(self):
+        data = self.get_state()
+        data = data.split("\n")[-1].split(",")
+        status = data[1]
+        if status == "CONNECTED":
+            local_ip = data[3]
+            remote_ip = data[4]
+        else:
+            local_ip = ""
+            remote_ip = ""
+
+        data = self.get_statistics().split("\n")
+        if len(data) > 6:
+            tun_read = data[2].split(",")[1]
+            tun_write = data[3].split(",")[1]
+            tcp_read = data[4].split(",")[1]
+            tcp_write = data[5].split(",")[1]
+        else:
+            tun_read = "0"
+            tun_write = "0"
+            tcp_read = "0"
+            tcp_write = "0"
+
+        return (status, local_ip, remote_ip, tun_read, tun_write, tcp_read, tcp_write)
+
+    def get_status(self):
+        data = self.get_state()
+        data = data.split("\n")[-1].split(",")
+        status = data[1]
+        return status
+        
+
 class vpn_control():
     def read_vpn_conf(self, dir, ext):
+        import glob
         vpn_conf = []
         for e in ext:
             l = glob.glob(dir+'/*.'+e)
@@ -269,7 +223,7 @@ class vpn_control():
     def __str__(self):
         s = ''
         for i,v in enumerate(self.vpn_list):
-            if v.get_status() != True: cmd = 'start'
+            if v.is_running() != True: cmd = 'start'
             else: cmd = 'stop'
             s = s+str(i+1)+'. '+v.name+' - '+cmd+'\n'
         return s   
@@ -284,7 +238,7 @@ class vpn_control():
             self.vpn_list.append(vpn(v,'/home/user/.openvpn-applet',auth_callback_fb=user_pass_dlg))
 
     def start(self, index,user=None,passwd=None,enable_log=False):
-        self.vpn_list[index].start(user,passwd)
+        return self.vpn_list[index].start(user,passwd)
         
     def stop(self, index):
         self.vpn_list[index].stop()
@@ -292,12 +246,12 @@ class vpn_control():
     def restart_connected(self):
         log("restart_connected")
         for vpn in self.vpn_list:
-            if vpn.get_status():
+            if vpn.is_running():
                 vpn.restart()
         
     def status(self, index):
         if len(self.vpn_list) > 0:
-            return self.vpn_list[index].get_status()
+            return self.vpn_list[index].is_running()
         else:
             return -1
 
@@ -307,7 +261,7 @@ class vpn_control():
     def get_active_connections_number(self):
         active = 0
         for i,v in enumerate(self.vpn_list):
-            if v.get_status(): active = active + 1
+            if v.is_running(): active = active + 1
         return active
 
     def get_vpn_name(self, number):
@@ -317,9 +271,9 @@ class vpn_control():
         return self.vpn_list[number].get_conf_file()
 
     
-class openvpn_applet(hildondesktop.StatusMenuItem):
+class openvpn_applet(StatusMenuItem):
     def __init__(self):
-        hildondesktop.StatusMenuItem.__init__(self)
+        StatusMenuItem.__init__(self)
 
         self.connection = conic.Connection()
         self.connection.set_property("automatic-connection-events", True)
@@ -329,18 +283,15 @@ class openvpn_applet(hildondesktop.StatusMenuItem):
         self.banner = None
         self.timer = None
         self.timeout = 0
-        #self.connection_text = "no connections"
         self.button_title_text = "OpenVPN"
 
         self.button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
-        #self.button.set_text(self.button_title_text, self.connection_text)
         self.button.set_style(hildon.BUTTON_STYLE_PICKER)
         self.button.set_alignment(0.2,0.5,1,1)
         image = gtk.image_new_from_icon_name("openvpn-applet", gtk.ICON_SIZE_BUTTON)
         self.button.set_image(image)
         self.button.set_image_position(gtk.POS_LEFT)
 
-        #self.button = gtk.Button("OpenVPN");
         self.button.connect("clicked", self.button_clicked_event)
         
         self.status_area_icon = gtk.gdk.pixbuf_new_from_file_at_size("/usr/share/icons/hicolor/48x48/apps/openvpn-applet.png", 18, 18)
@@ -349,8 +300,6 @@ class openvpn_applet(hildondesktop.StatusMenuItem):
         self.show_all()
         self.update_list()
         self.update_button()
-        #self.set_settings(True)
-        #self.connect("show-settings", self.open_settings)
      
     def update_list(self,source=None):
         if self.vc == None:
@@ -384,9 +333,9 @@ class openvpn_applet(hildondesktop.StatusMenuItem):
         self.timeout = self.timeout - 1
         self.update_button()
         if self.timeout > 0:
-            self.timer = gobject.timeout_add(500, self.update_button_timer)
+            self.timer = timeout_add(500, self.update_button_timer)
         elif self.vc.get_active_connections_number() > 0:
-            self.timer = gobject.timeout_add(30000, self.update_button_timer)
+            self.timer = timeout_add(30000, self.update_button_timer)
         else:
             self.timer = None
             return False
@@ -397,12 +346,9 @@ class openvpn_applet(hildondesktop.StatusMenuItem):
         return widget
         
     def open_settings(self, widget=None, data=None):
-        #self.set_options_visible(False)
         dlg = OpenvpnAppletSettings(vpn_manager=self.vc)
         dlg.run()
         dlg.destroy()
-        #self.set_options_visible(True)
-        #self.combo.set_active(0)
         self.update_button()
 
     def manage_connection(self, widget, vpn_number):
@@ -413,12 +359,16 @@ class openvpn_applet(hildondesktop.StatusMenuItem):
         if vpn_number >= 0  and vpn_number < self.vc.get_vpn_count():
             if self.vc.status(vpn_number) == False:
                 self.banner = hildon.hildon_banner_show_information(self, "", "Starting Openvpn Connection")
-                self.vc.start(vpn_number,None,None)
+                (status, message) = self.vc.start(vpn_number,None,None)
+                if status != "SUCCESS":
+                    dlg = hildon.hildon_note_new_information(gtk.Window(), message)
+                    dlg.run()
+                    dlg.destroy()
             else:
                 self.banner = hildon.hildon_banner_show_information(self,"", "Stopping Openvpn Connection")
                 self.vc.stop(vpn_number)
             self.timeout = 10
-            self.timer = gobject.timeout_add(500, self.update_button_timer)
+            self.timer = timeout_add(500, self.update_button_timer)
         elif vpn_number == self.vc.get_vpn_count():
             log("open settings")
             self.open_settings(self)
@@ -426,7 +376,7 @@ class openvpn_applet(hildondesktop.StatusMenuItem):
     def connection_cb(self, connection, event):
         log("connection_cb",connection, event)
         
-        status = event.get_status()
+        status = event.is_running()
         error = event.get_error()
         iap_id = event.get_iap_id()
         bearer = event.get_bearer_type()
@@ -474,7 +424,7 @@ class SelectConnectionDialog(gtk.Dialog):
         for vpn_number,v in enumerate(vpn_manager.vpn_list):
             hbox = gtk.HBox()
             button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
-            if v.get_status():
+            if v.is_running():
                 stats_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
                 stats_button.set_text("Status",'')
                 stats_button.connect("clicked", self.stats_button_clicked_event, vpn_number)
@@ -498,14 +448,13 @@ class SelectConnectionDialog(gtk.Dialog):
         self.action_area.show_all()
         self.vbox.show_all()
         self.set_size_request(800,vbox.size_request()[1]+10)
-        #self.show_all()
 
     def button_clicked_event(self, widget, vpn_number):
         self.response(vpn_number)
         self.destroy()
 
     def stats_button_clicked_event(self, widget, vpn_number):
-        dlg = OpenvpnAppletTestConnection(parent=self, title=self.vpn_manager.get_vpn_name(vpn_number), vpn_conn=self.vpn_manager.vpn_list[vpn_number])            
+        dlg = OpenvpnAppletConnectionStatus(parent=self, title=self.vpn_manager.get_vpn_name(vpn_number), vpn_conn=self.vpn_manager.vpn_list[vpn_number])            
         dlg.run()        
         dlg.destroy()
 
@@ -520,6 +469,7 @@ class OpenvpnAppletSettings(gtk.Dialog):
         self.new_button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.new_button.set_label("New")
         self.new_button.connect("clicked", self.open_new_dialog)
+        self.new_button.set_size_request(174,-1)
         self.delete_button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.delete_button.set_label("Delete")
         self.delete_button.connect("clicked", self.delete_clicked_event)
@@ -572,11 +522,10 @@ class OpenvpnAppletSettings(gtk.Dialog):
                 mstr = ""
                 for f in files_to_overwrite_message:
                     mstr = mstr+"\n"+f
-                mdlg = gtk.MessageDialog(dlg,0,gtk.MESSAGE_QUESTION,gtk.BUTTONS_NONE,"Overwrite files?\n"+mstr)
-                mdlg.add_buttons("Overwrite", gtk.RESPONSE_ACCEPT,"Don't Overwrite", gtk.RESPONSE_NONE)
+                mdlg = hildon.hildon_note_new_confirmation(dlg,"Overwrite files?\n"+mstr)
                 response = mdlg.run()
                 mdlg.destroy()
-                if response == gtk.RESPONSE_ACCEPT:
+                if response == gtk.RESPONSE_OK:
                     files_to_copy = all_files
             else:
                 files_to_copy = all_files
@@ -597,11 +546,10 @@ class OpenvpnAppletSettings(gtk.Dialog):
             mstr = filename
             for f in file_links[filename]:
                 mstr += "\n"+str(f)
-            dlg = gtk.MessageDialog(self,0,gtk.MESSAGE_QUESTION,gtk.BUTTONS_NONE,"Deleting files:\n"+mstr)
-            dlg.add_buttons("Delete", gtk.RESPONSE_ACCEPT,"Cancel", gtk.RESPONSE_NONE)
+            dlg = hildon.hildon_note_new_confirmation(self,"Delete files:\n"+mstr)
             response = dlg.run()
             dlg.destroy()
-            if response == gtk.RESPONSE_ACCEPT:
+            if response == gtk.RESPONSE_OK:
                 delete_files(file_links[filename])
                 delete_files([filename])
         else:
@@ -614,14 +562,18 @@ class OpenvpnAppletSettings(gtk.Dialog):
         if vpn_number >= 0:
             log("Item selected", vpn_number)
             vpn = self.vpn_manager.vpn_list[vpn_number]
-            if vpn.get_status():
+            if vpn.is_running():
+                vpn.stop()
+            (status, message) = vpn.start(None,None,None)
+            if status != "SUCCESS":
+                dlg = hildon.hildon_note_new_information(gtk.Window(), message)
+                dlg.run()
+                dlg.destroy()
+            else:
+                dlg = OpenvpnAppletTestConnection(parent=self, vpn_conn=vpn)            
+                dlg.run()        
+                dlg.destroy()
                 vpn.stop()
-            vpn.start(None,None,True)
-
-            dlg = OpenvpnAppletTestConnection(parent=self, vpn_conn=vpn)            
-            dlg.run()        
-            dlg.destroy()
-            vpn.stop()
 
     def about_clicked_event(self,widget,data=None):
         from about import HeAboutDialog
@@ -640,6 +592,67 @@ class OpenvpnAppletSettings(gtk.Dialog):
             self.destroy()
 
 
+class OpenvpnAppletConnectionStatus(gtk.Dialog):
+    def __init__(self,title="Connection Status", parent=None, flags=0, buttons=None,vpn_conn=None):
+        gtk.Dialog.__init__(self,title, parent, flags, buttons)
+        self.vpn_conn = vpn_conn        
+
+        status_label = gtk.Label("Status:")
+        local_ip_label = gtk.Label("Local IP:")
+        remote_ip_label = gtk.Label("Remote IP:")
+        tun_read_label = gtk.Label("TUN/TAP read bytes:")
+        tun_write_label = gtk.Label("TUN/TAP write bytes:")
+        tcp_read_label = gtk.Label("TCP/UDP read bytes:")
+        tcp_write_label = gtk.Label("TCP/UDP write bytes:")
+
+        vbox = gtk.VBox()
+        vbox.pack_start(status_label)
+        vbox.pack_start(local_ip_label)
+        vbox.pack_start(remote_ip_label)
+        vbox.pack_start(tun_read_label)
+        vbox.pack_start(tun_write_label)
+        vbox.pack_start(tcp_read_label)
+        vbox.pack_start(tcp_write_label)
+
+        hbox = gtk.HBox()
+        hbox.pack_start(vbox)
+
+        (status, local_ip, remote_ip, tun_read, tun_write, tcp_read, tcp_write) = self.vpn_conn.get_current_status()
+        self.status_label = gtk.Label(status)
+        self.local_ip_label = gtk.Label(local_ip)
+        self.remote_ip_label = gtk.Label(remote_ip)
+        self.tun_read_label = gtk.Label(tun_read)
+        self.tun_write_label = gtk.Label(tun_write)
+        self.tcp_read_label = gtk.Label(tcp_read)
+        self.tcp_write_label = gtk.Label(tcp_write)
+
+        vbox = gtk.VBox()
+        vbox.pack_start(self.status_label)
+        vbox.pack_start(self.local_ip_label)
+        vbox.pack_start(self.remote_ip_label)
+        vbox.pack_start(self.tun_read_label)
+        vbox.pack_start(self.tun_write_label)
+        vbox.pack_start(self.tcp_read_label)
+        vbox.pack_start(self.tcp_write_label)
+
+        hbox.pack_start(vbox)
+
+        pannable_area = hildon.PannableArea()
+        pannable_area.add_with_viewport(hbox)
+        self.vbox.pack_start(pannable_area)
+
+        button = self.add_button("Details", gtk.RESPONSE_NONE)
+        button.connect("clicked", self.details_button_clicked)
+
+        self.set_size_request(800,240)
+
+        self.show_all()
+    
+    def details_button_clicked(self, widget, data=None):
+        dlg = OpenvpnAppletTestConnection(parent=self, title=self.vpn_conn.get_name(), vpn_conn=self.vpn_conn)            
+        dlg.run()        
+        dlg.destroy()
+
 class OpenvpnAppletTestConnection(gtk.Dialog):
     def __init__(self,title="Test Connection", parent=None, flags=0, buttons=None,vpn_conn=None):
         gtk.Dialog.__init__(self,title, parent, flags, buttons)
@@ -684,7 +697,7 @@ class OpenvpnAppletTestConnection(gtk.Dialog):
         
         self.update_text()
         self.show_all()
-        self.timer = gobject.timeout_add(5000, self.update_text)
+        self.timer = timeout_add(5000, self.update_text)
     
     def update_text(self):
         log("update_text")
@@ -784,6 +797,7 @@ class OpenvpnAppletNewConnection(gtk.Dialog):
     
     def browse_button_event(self, widget, data=None):
         dlg = hildon.FileChooserDialog(self, gtk.FILE_CHOOSER_ACTION_OPEN, hildon.FileSystemModel())
+        dlg.set_current_folder(os.path.expanduser("~/MyDocs"))
         response = dlg.run()
         if response == gtk.RESPONSE_OK:
             widget.set_value(dlg.get_filename())
@@ -805,7 +819,7 @@ class OpenvpnAppletNewConnection(gtk.Dialog):
                 for f in filelinks.get_not_found():
                     if isinstance(f,str):
                         mstr = mstr+"\n"+f
-                mdlg = gtk.MessageDialog(self,0,gtk.MESSAGE_INFO,gtk.BUTTONS_OK,"Files not found:\n"+mstr)
+                mdlg = hildon.hildon_note_new_information(gtk.Dialog(),"Files not found:\n"+mstr)
                 response = mdlg.run()
                 mdlg.destroy() 
         else:
@@ -845,6 +859,8 @@ class FileLinks(UserDict):
             self.data[self.path] = self.__scan(self.path, os.path.dirname(self.path))
         
     def __scan(self,file,root):
+        import fileinput
+        from sets import Set
         log("function: __scan", file, root)
         fileset = Set()
         for line in fileinput.input(file):
@@ -923,12 +939,6 @@ class FileLinks(UserDict):
     
 hd_plugin_type = openvpn_applet
 
-def log(*input):
-    if enable_logging:
-        print input
-        print >>fd,input
-        fd.flush()
-
 # The code below is just for testing purposes.
 # It allows to run the widget as a standalone process.
 if __name__ == "__main__":
